;;; x-symbol-B.el --- token language "B entity" for package x-symbol
;; 
;;   by Dominique Cansell from 
;;
;;; x-symbol-tla.el --- token language "TLA entity" for package x-symbol
;; 
;; Copyright (C) 1996-1999, 2002, 2003 Free Software Foundation, Inc.
;;
;; Author: Christoph Wedler <wedler@users.sourceforge.net>
;; Maintainer: (Please use `M-x x-symbol-package-bug' to contact the maintainer)
;; Version: 4.5
;; Keywords: WYSIWYG, HTML, wp, math, internationalization
;; X-URL: http://x-symbol.sourceforge.net/

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; If you want to use package x-symbol, please visit the URL (use
;; \\[x-symbol-package-web]) and read the info (use \\[x-symbol-package-info]).

;; Token language B is registered in x-symbol-hooks.

;;; Code:

(provide 'x-symbol-B)

;;;===========================================================================
;;;  Auto-style
;;;===========================================================================

(defcustom x-symbol-B-auto-style
  '((not (memq major-mode '(pri-machines-mode)))
    nil   ;; x-symbol-coding
    'null ;; x-symbol-8bits	   [NEVER want it; null disables search]
    nil   ;; x-symbol-unique
    t     ;; x-symbol-subscripts
    nil)  ;; x-symbol-image
  "Variable used to document a language access.
See documentation of `x-symbol-auto-style'."
  :group 'x-symbol-B
  :group 'x-symbol-mode
  :type 'x-symbol-auto-style)

;; FIXME da: is this needed?
(defcustom x-symbol-B-auto-coding-alist nil
  "*Alist used to determine the file coding of B buffers.
Used in the default value of `x-symbol-auto-mode-alist'.  See
variable `x-symbol-auto-coding-alist' for details."
  :group 'x-symbol-B
  :group 'x-symbol-mode
  :type 'x-symbol-auto-coding)

;;;===========================================================================
;;;  Miscellaneous variables
;;;===========================================================================

;;(defface x-symbol-B-symbol-face
;;  '((((class color) (background light))
;;     (:foreground "orange4")))
;;  "*Face used for entities with name representing non-Latin-1 characters.
;;Used in `x-symbol-B-class-face-alist'."
;;  :group 'x-symbol-B
;;  :group 'x-symbol-info-general)

;;(defface x-symbol-B-noname-face
;;  '((((class color) (background light))
;;     (:foreground "red4")))
;;  "*Face used for Latin-N character entities without name.
;;Used in `x-symbol-B-class-face-alist'."
;;  :group 'x-symbol-B
;;  :group 'x-symbol-info-general)

(defcustom x-symbol-B-modeline-name "B"
  "Modeline name of token language `B'.
See language access `x-symbol-LANG-modeline-name'."
  :group 'x-symbol-B
  :type 'string)

(defcustom x-symbol-B-header-groups-alist nil
;;  '(("Operator" bigop operator)
;;    ("Relation" relation)
;;    ("Arrow, Punctuation" arrow triangle shape
;;     white line dots punctuation quote parenthesis)
;;    ("Symbol" symbol currency mathletter setsymbol)
;;    ("Greek Letter" greek greek1)
;;    ("Misc. Letter" letter slash)
;;    ("Cedilla, Ogonek" cedilla ogonek)
;;    ("Dotaccent, Ring" dotaccent ring)
;;    ("Tilde, Breve" tilde breve)
;;    ("Circumflex, Caron" circumflex caron)
;;    ("Diaeresis, Umlaut" diaeresis hungarumlaut)
;;    ("Acute, Grave" acute grave))
  "Header/submenu specification of the specific menu for language `B'.
See language access `x-symbol-LANG-header-groups-alist'."
  :group 'x-symbol-B
  :group 'x-symbol-input-init
  :type 'x-symbol-headers)

(defcustom x-symbol-B-class-alist
  '((symbol)
    (noname "B char-ref" (x-symbol-emph-info-face))
    (VALID "B entity" (x-symbol-info-face))
    (INVALID "no B entity" (x-symbol-emph-info-face)))
  "Token classes displayed by info in echo area, for language `B'.
See language access `x-symbol-LANG-class-alist'."
  :group 'x-symbol-B
  :group 'x-symbol-info-strings
  :type 'x-symbol-class-info)

;;(defcustom x-symbol-B-class-face-alist
;;  '((symbol x-symbol-B-symbol-face (x-symbol-B-symbol-face))
;;    (noname x-symbol-B-noname-face (x-symbol-B-noname-face)))
;;  "Color scheme in language specific grid and info, for language `B'.
;;See language access `x-symbol-LANG-class-face-alist'."
;;  :group 'x-symbol-B
;;  :group 'x-symbol-input-init
;;  :group 'x-symbol-info-general
;;  :type 'x-symbol-class-faces)

(defcustom x-symbol-B-electric-ignore nil
  ;;"::"
  "Specification restricting input method ELECTRIC with language `B'.
See language access `x-symbol-LANG-electric-ignore'."
  :group 'x-symbol-B
  :group 'x-symbol-input-control
  :type 'x-symbol-function-or-regexp)

(defvar x-symbol-B-token-list 'x-symbol-B-token-list-name
  "Symbol specifying the token definition for language `B'.
Allowed values are
 - `x-symbol-B-token-list-name': the canonical token for a character
   is a entity references,
 - `x-symbol-B-token-list-code': the canonical token for a character
   is a character references,
 - `x-symbol-B-token-list-netscape': the canonical token for a
   Latin-1 character is a entity references, for others, it is a
   character references.  Bug workaround for Netscape, v4.6 or lower.

The value is used by function `x-symbol-B-default-token-list' which
is used for the definition of the conversion tables.  See
`x-symbol-B-token-grammar'.")

(defvar x-symbol-B-token-list 'identity)

(defvar x-symbol-B-token-grammar
  '(x-symbol-make-grammar
    :decode-regexp "\\([0-9A-Za-z_]+\\)\\|~\\|:|\\|||\\|\\\.\\\.\\|<[:|\+-=>]*\\|>=\\|<=>\\|=>\\|<->\\||->\\|-->[>]?\\|>->[>]?\\|>[+]>[>]?\\|+->[>]?\\||--\\|\\(/?[<:=]+\\)\\|{}\\|><\\|<|\\|<<[:|]?\\||[>]+\\|\\(/[<:]+\\)\\|/<<\\|/\\\\\\|\\\\/\\|\\([>&+-*%!#:]+\\)"



;<=>\\|=>\\|<->\\|=\\|==\\|{}\\|><\\|\\([<>~\\/&+-*%!#:|]+\\)"

;;"\\([0-9A-Za-z_]+\\)\\|-->\\|/=\\|&\\|!\\|#|\\<:\\|<\|\\|\|>/\\|<=>\\|<->\\|>->\\|>+>\\|{}\\|\\/\\|/\\\\|=>\\([><~&%!#{}:]\\)"

;; <:\|<<:\\|/:\\|<|\\||>\\|\|->\\|<->\\|<=>\\|{}\\|/\\\\\\|\\\\/\\|

;;"\\([\\\\_][0-9A-Za-z]+\\)\\|\\(#\\)\\\|-->\\|<:\\|<->\\|>->\\|/\\\\|>+>\\+->\\|\\([=<>-\\/'|~]+\\)\\|\\({}\\)\\|\\(\\[\\]\\)\\|\\(([+\\.\\X/]+)\\)\\|\\(NATURAL\\|INTEGER\\|/:\\|:\\| or \\|&\\|!\\|%\\)"
    :token-list x-symbol-B-default-token-list)
  "Grammar of token language `B'.
See language access `x-symbol-LANG-token-grammar'.")

(defvar x-symbol-B-user-table nil
  "User table defining B entities, used in `x-symbol-B-table'.")

(defvar x-symbol-B-generated-data nil
  "Generated data for token language `B'.
See language access `x-symbol-LANG-generated-data'.")


;;;===========================================================================
;;;  Super- and Subscripts
;;;===========================================================================


(defcustom x-symbol-B-subscript-matcher 'x-symbol-B-subscript-matcher
  "Function matching super-/subscripts for language `B'.
See language access `x-symbol-LANG-subscript-matcher'."
  :group 'x-symbol-B
  :type 'function)

(defcustom x-symbol-B-font-lock-allowed-faces
  '(B-math-face
    font-lock-string-face font-lock-doc-string-face font-latex-math-face)
  "*Faces which are allowed when fontifying simple super- and subscripts.
Package x-symbol only uses super- and subscripts if they are in braces,
if the \"^\"/\"_\" has not been fontified yet or is only fontified with
faces which appear in this list.  Value t means, always use super- and
subscripts."
  :group 'x-symbol-B
  :type '(repeat (symbol :tag "Face name"))) ; face would create faces... :-(

(defvar x-symbol-B-font-lock-regexp "~"
  "Regexp matching the prefix of super-/subscripts.
The first regexp group should match the super-/subscript command.")

(defvar x-symbol-B-font-lock-limit-regexp "."
  "Regexp matching the limit for the end of super-/subscripts.
This regexp should match the end of line.")

(defcustom x-symbol-B-font-lock-regexp ""
  "Regexp matching the start tag of B's super- and subscripts.
See also `x-symbol-B-font-lock-alist'."
  :group 'x-symbol-B
  :type 'regexp)

(defcustom x-symbol-B-font-lock-limit-regexp "\n\\|\s\\|.\\|\t"
  "Regexp matching the end tag of B's super- and subscripts.
This regexp should match the end of line and the closing tags in
`x-symbol-B-font-lock-alist'."
  :group 'x-symbol-B
  :type 'regexp)

(defcustom x-symbol-B-font-lock-contents-regexp "\s"
  "*Regexp matching the super- and subscript contents.
This regexp should match the text between the opening and closing super-
or subscript tag."
  :group 'x-symbol-B
  :type 'regexp)

(defcustom x-symbol-B-font-lock-alist
  nil ;;'(("~" . " "))
  "Alist for correct tag pairs for HTML's super- and subscripts.
Each element looks like (OPEN . CLOSE).  All keys OPEN in this alist
should be matched by `x-symbol-B-font-lock-regexp', all CLOSEs should
be matched by `x-symbol-B-font-lock-limit-regexp'."
  :group 'x-symbol-B
  :type '(repeat (cons :format "%v"
		       (string :tag "Open tag")
		       (string :tag "Close tag"))))

;;;===========================================================================
;;;  The tables
;;;===========================================================================




(defun x-symbol-B-default-token-list (tokens)
  (mapcar #'list
	  (and (car tokens)
	       (memq x-symbol-B-token-list
		     '(x-symbol-B-token-list-name
		       x-symbol-B-token-list-code
		       x-symbol-B-token-list-netscape))
	       (if (or (eq x-symbol-B-token-list
			   'x-symbol-B-token-list-name)
		       (and (eq x-symbol-B-token-list
				'x-symbol-B-token-list-netscape)
			    (< (car tokens) 256)))
		   (append (cdr tokens) (list (format "&#%d;" (car tokens))))		 (cons (format "&#%d;" (car tokens)) (cdr tokens))))))



(defvar x-symboll-B-latin1-table
  '((notsign  () 172 "not"))
  "Table defining B entities, see `x-symbol-B-table'.")

(defvar x-symbol-B-xsymb0-table
  '((Delta (symbol) 916 "Delta")
    (Phi (symbol) 934 "Phi")
    (Gamma (symbol) 915 "Gamma")
    (theta1 (symbol) 977 "thetasym")
    (Lambda (symbol) 923 "Lambda")
    (Pi (symbol) 928 "Pi")
    (Theta (symbol) 920 "Theta")
    (Sigma (symbol) 931 "SIGMA")
    (sigma1 (symbol) 962 "sigmaf")
    (Omega (symbol) 937 "Omega")
    (Xi (symbol) 926 "Xi")
    (Psi (symbol) 936 "Psi")
    (alpha (symbol) 945 "alpha")
    (beta (symbol) 946 "beta")
    (chi (symbol) 967 "chi")
    (delta (symbol) 948 "delta")
    (epsilon (symbol) 949 "epsilon")
    (phi (symbol) 966 "phi")
    (gamma (symbol) 947 "gamma")
    (eta (symbol) 951 "eta")
    (iota (symbol) 953 "iota")
    (kappa (symbol) 954 "kappa")
    (lambda (symbol) 955 "%")
    (mu (symbol) 956 "mu")
    (nu (symbol) 957 "nu")
    (pi (symbol) 960 "pi")
    (theta (symbol) 952 "theta")
    (rho (symbol) 961 "rho")
    (sigma (symbol) 963 "sigma")
    (tau (symbol) 964 "tau")
    (upsilon (symbol) 965 "upsilon")
    (omega1 (symbol) 982 "piv")
    (omega (symbol) 969 "omega")
    (xi (symbol) 958 "xi")
    (psi (symbol) 968 "psi")
    (zeta (symbol) 950 "zeta")
    (Upsilon1 (symbol) 978 "<<|")
    (spade (symbol) 979 ">->>")
    (florin (symbol) 402 ">->")
;    (bullet (symbol) 8226 "\\bullet")
;    (ellipsis (symbol) 8230 "\\hellip")
    (minute (symbol) 8242 "|>>")
    (second (symbol) 8243 "::")
;    (radicalex (symbol) 8254 "\\oline")
;    (fraction (symbol) 8260 "\\frasl")
    (weierstrass (symbol) 8472 "POW")
    (Ifraktur (symbol) 8465 "FIN1")
    (Rfraktur (symbol) 8476 "POW1")
  ;  (trademark (symbol) 8482 "FIN1")
    (aleph (symbol) 8501 "FIN")
    (arrowleft (symbol) 8592 "<-")
 ;   (arrowup (symbol) 8593 "\\uarr")
    (arrowright (symbol) 8594 "-->")
 ;   (arrowdown (symbol) 8595 "\\darr")
   (arrowboth (symbol) 8596 "<->")
   (carriagereturn (symbol) 8629 "NATURAL1")
   ; (arrowdblleft (symbol) 8656 "\\lArr")
 ;   (arrowdblup (symbol) 8657 "\\uArr")
    (arrowdblright (symbol) 8658 "=>")
 ;   (arrowdbldown (symbol) 8659 "\\dArr")
    (arrowdblboth (symbol) 8660 "<=>")
   ;; (char_0xac (symbol) 8696 "\res")
   (partialdiff (symbol) 8706 "||")
    (emptyset (symbol) 8709 "{}")
   ; (gradient (symbol) 8711 "\\nabla")
    (notelement (symbol) 8713 "/:")
    (element (symbol) 8712 ":")
   ; (suchthat (symbol) 8715 "\\ni")
 ;   (product (symbol) 8719 "\\\*")
  ;  (summation (symbol) 8721 "\\sum")
  ;  (minus1 (symbol) 8722 "\\minus")
  ;  (asterisk1 (symbol) 8727 "\\star")
  ;  (radical (symbol) 8730 "\\radic")
   (proportional (symbol) 8733 "~")
    (infinity (symbol) 8734 "<+")
  ;  (angle (symbol) 8736 "\\ang")
    (logicaland (symbol) 8743 "\&")
    (logicalor (symbol) 8744 "or")
    (intersection (symbol) 8745 "/\\")
    (union (symbol) 8746 "\\/")
 ;   (sqintersection (symbol) 8851 "\\sqcap")
 ;   (squnion (symbol) 8852 "\\sqcup")
 ;   (integral (symbol) 8747 "\\int")
 ;   (similar (symbol) 8764 "\\sim")
 ;   (simequal (symbol) 8771 "\\simeq")
 ;   (congruent (symbol) 8773 "\\cong")
    (notequal (symbol) 8800  "/=")
    (equivalence (symbol) 8801 ":=")
    (lessequal (symbol) 8804 "<=")
    (greaterequal (symbol) 8805 ">=")
 ;   (properprec (symbol) 8826 "\\prec")
 ;   (propersucc (symbol) 8827 "succ")
    (reflexprec (symbol) 8828 "\\preceq")
 ;   (reflexsucc (symbol) 8829 "\\succeq")
    (propersubset (symbol) 8834 "<<:")
 ;   (propersuperset (symbol) 8835 "\\supset")
 ;   (unionplus (symbol) 8846 "\\uplus")
 ;   (propersqsubset (symbol) 8847 "\\sqsubset")
 ;   (propersqsuperset (symbol) 8848 "\\sqsupset")
    (notsubset (symbol) 8836 "/<<:")
    (reflexsubset (symbol) 8838 "<:")
    (reflexsuperset (symbol) 8839 "/<:")
 ;   (reflexsqsubset (symbol) 8849 "\\sqsubseteq")
 ;   (reflexsqsuperset (symbol) 8850 "\\sqsupseteq")
   ;;(circleplus (symbol) 8853 "><")
  ;  (circleminus (symbol) 8854 "\\ominus" "(-)")
   (circlemultiply (symbol) 8855 "><")
  ;  (circleslash (symbol) 8856 "\\oslash" "(/)")
  ;  (circledot (symbol) 8857 "\\odot" "(.)")
    (bardash (symbol) 8866 "|--")
 ;   (dashbar (symbol) 8867 "-|")
 ;   (bardashdbl (symbol) 8872 "|=")
 ;   (perpendicular (symbol) 8869 "\\perpendicular")
 ;   (periodcentered1 (symbol) 8901 "\\cdot")
  ;;  (angleleft (symbol) 9001 "<<")
  ;;  (angleright (symbol) 9002 ">>")
  ;  (lozenge1 (symbol) 9671 "<>")
  ;  (box (symbol) 9633 "[]")
  ;  (bigcircle (symbol) 9675 "\\bigcirc")
  ;  (circ (symbol) 9702 "\\circ" "\\o")
  ;  (star (symbol) 9733 "\\star")
  ;  (spade (symbol) 9824 "\\spades")
    (club (symbol) 9827 "-->>")
    (heart (symbol) 9829 "+->>")
    (diamond (symbol) 9830 "+->")
 (ring (symbol) 9831  ">+>")
 (plusminus1  (symbol) 9832  ">+>>")
 (ellipsis (symbol) 9833 "..")
 (daggerdbl (symbol) 9834 "/<:")
 ;(parenlefttp (symbol) 9835 "union")
 ;(parenleftex (symbol) 9836 "inter")
 (arrowhorizex (symbol) 9837 "NATURAL")
)
  "Table defining B entities, see `x-symbol-B-table'.")

(defvar x-symbol-B-xsymb1-table
  '(;(ampersand2 () 38 "\\amp")
    ;(quotedbl1 () 34 "\\quot")
    (mapsto (symbol) 101 "|->")
    (less2 () 60 "<")
    (greater2 () 62 ">")
    (universal1 (symbol) 8704 "!")
    (existential1 (symbol) 8707 "#")
   ; (circumflex (symbol) 710 "_circ")
   ; (tilde (symbol) 732 "\\tilde")
   ; (trianglerighteq (symbol) 10852 "\\rest")
    (triangleright  (symbol) 123 "|>")
    (triangleleft  (symbol) 123 "<|")
   ;;(ensp (symbol) 8194 "\\ensp") ;  en space, U+2002 ISOpub
    ;;(emsp (symbol) 8195 "\\emsp") ;  em space, U+2003 ISOpub
    ;;(thinsp (symbol) 8201 "\\thinsp") ;  thin space, U+2009 ISOpub
    ;;(zwnj (symbol) 8204 "\\zwnj") ;  zero width non-joiner, U+200C NEW RFC 2070
    ;;(zwj (symbol) 8205 "\\zwj") ;  zero width joiner, U+200D NEW RFC 2070
    ;;(lrm (symbol) 8206 "\\lrm") ;  left-to-right mark, U+200E NEW RFC 2070
    ;;(rlm (symbol) 8207 "\\rlm") ;  right-to-left mark, U+200F NEW RFC 2070
  ;  (endash (symbol) 8211 "\\ndash")
  ;  (emdash (symbol) 8212 "\\mdash")
    ;;(lsquo (symbol) 8216 "\\lsquo") ;  left single quotation mark, U+2018 ISOnum
    ;;(rsquo (symbol) 8217 "\\rsquo") ;  right single quotation mark, U+2019 ISOnum
    ;;(sbquo (symbol) 8218 "\\sbquo") ;  single low-9 quotation mark, U+201A NEW
    ;;(ldquo (symbol) 8220 "\\ldquo") ;  left double quotation mark, U+201C ISOnum
    ;;(rdquo (symbol) 8221 "\\rdquo") ;  right double quotation mark, U+201D ISOnum
    ;;(bdquo (symbol) 8222 "\\bdquo") ;  double low-9 quotation mark, U+201E NEW
  ;  (dagger (symbol) 8224 "\\dagger")
  ;  (daggerdbl (symbol) 8225 "\\Dagger")
  ;  (perthousand (symbol) 8240 "\\permil")
  ;  (guilsinglleft (symbol) 8249 "\\lsaquo")
  ;  (guilsinglright (symbol) 8250 "\\rsaquo")
  ;  (therefore1 (symbol) 8756 "\\there4")
  ;  (ceilingleft (symbol) 8968 "\\lceil")
  ;  (ceilingright (symbol) 8969 "\\rceil")
  ;  (floorleft (symbol) 8970 "\\lfloor")
  ;  (floorright (symbol) 8971 "\\rfloor")
  ;  (asym (symbol) 8776 "\\asymp")
    ;(dotequal (symbol) 8784 "\\doteq")
    (approxequal (symbol) 8776 ":|")
    (definedas (symbol) 8796 "==")
 ;   (leadsto (symbol) 8594 "~>")
   ; (natnums (symbol) 8469 "NATURAL")
   ;; (realnums (symbol) 8477 "real")
    (integers (symbol) 120171 "INTEGER")
   (bigintersection  (symbol)  8776 "INTER")
    (bigunion  (symbol) 8784 "UNION")
    (lessless  (symbol) 8785 "<<")
    )
  "Table defining B entities, see `x-symbol-B-table'.")


(defvar x-symbol-B-table
  (append x-symbol-B-user-table
	  '(nil)
          x-symboll-B-latin1-table
	  x-symbol-B-xsymb0-table
	  x-symbol-B-xsymb1-table)
  "Table defining `B' tokens for the characters.
See language access `x-symbol-LANG-table' and variable
`x-symbol-B-token-list'.  Use `x-symbol-B-user-table' to define
private B entities or shadow existing ones.")


;;===========================================================================
;;  Subscript functions
;;===========================================================================
(defun x-symbol-B-subscript-matcher (limit)
;;  checkdoc-params: (limit)
  "Match and skip over super- and subscripts.
Return nil if `x-symbol-mode' or `x-symbol-subscripts' is nil.  Uses
`x-symbol-B-font-lock-regexp'."
  (block nil
    (let (open-beg open-end close-end close-beg)
      (while (re-search-forward x-symbol-B-font-lock-regexp limit t)
	(setq open-beg (match-beginning 0)
	      open-end (match-end 0))
	(when (re-search-forward x-symbol-B-font-lock-limit-regexp
				 limit 'limit)
	  (setq close-beg (match-beginning 0)
		close-end (match-end 0))
	  (if (equal (cdr (assoc (downcase
				  (buffer-substring open-beg open-end))
				 x-symbol-B-font-lock-alist))
		     (downcase (buffer-substring close-beg close-end)))
	      (when
		  (save-excursion
		    (goto-char open-end)
		    (re-search-forward x-symbol-B-font-lock-contents-regexp
				       close-beg t))
		(store-match-data (list open-beg close-end
					open-beg open-end
					open-end close-beg
					close-beg close-end))
		(return (if (eq (char-after (+ 3 open-beg)) ?b)
			    'x-symbol-sub-face
			  'x-symbol-sup-face)))
	    (goto-char close-beg)))))))
;; Local IspellPersDict: .ispell_xsymb
;; x-symbol-B.el ends here